Name: ipmiutil
%define Version 1.5.8
Version: %Version
Release: 1
Summary: A package that includes various IPMI server management utilities.
Copyright: BSD
Group: System Environment/Kernel
Source: ipmiutil-%Version.tar.gz
#Requires: ipmidrvr
%ifarch x86_pentium3
AutoReqProv: No
%endif
%ifarch x86_pentium4
AutoReqProv: No
%endif

%description
The ipmiutil component package provides utilities to view the SEL (showsel), 
perform a hardware reset (hwreset), and set up the Platform Event Filter 
entry to allow BMC LAN alerts from OS Critical Stop messages (pefconfig).  
It requires an IPMI driver (ipmidrvr) package in order to talk to the 
BMC/firmware interface.
An IPMI driver can be provided by either the Intel IPMI driver (/dev/imb) 
or the valinux IPMI driver (/dev/ipmikcs).


%prep
%setup -q

%build
sh configure
make

%install
RPM_BUILD_ROOT=`pwd`
make DESTDIR=${RPM_BUILD_ROOT} install
( cd ${RPM_BUILD_ROOT}/usr/share/man/man8; gzip -f *.8 ) 

%files
%defattr(0755,root,root)
/usr/sbin/showsel
/usr/sbin/hwreset
/usr/sbin/sensor
/usr/sbin/pefconfig
/usr/sbin/tmconfig
/usr/sbin/fruconfig
/usr/sbin/alarms
/usr/sbin/wdt
/usr/sbin/icmd
/usr/share/ipmiutil/checksel
/usr/share/ipmiutil/libfreeipmi.so
%defattr(0664,root,root)
/usr/share/ipmiutil/README
/usr/share/ipmiutil/COPYING
/usr/share/ipmiutil/UserGuide
/usr/share/ipmiutil/bmclanpet.mib
/usr/share/ipmiutil/bmclanaol.mib
/usr/share/man/man8/showsel.8.gz
/usr/share/man/man8/sensor.8.gz
/usr/share/man/man8/hwreset.8.gz
/usr/share/man/man8/alarms.8.gz
/usr/share/man/man8/wdt.8.gz
/usr/share/man/man8/pefconfig.8.gz
/usr/share/man/man8/tmconfig.8.gz
/usr/share/man/man8/fruconfig.8.gz
/usr/share/man/man8/icmd.8.gz
# %defattr(-,root,root)
# %doc README TODO COPYING ChangeLog

%clean
RPM_BUILD_ROOT=`pwd`
# rm -rf ${RPM_BUILD_ROOT}

%pre
# before install
pdir=/usr/share/ipmiutil
echo "Installing IPMI Management Utilities %Version ..."
mkdir -p $pdir

dev1=/dev/imb
dev2=/dev/ipmi0
dev3=/dev/ipmikcs
if [ ! -c $dev1 ]
then
  if [ ! -c $dev2 ]
  then
     if [ ! -c $dev3 ]
     then
        echo "Warning: No IPMI devices found ($dev1, $dev2, or $dev3)."
        # echo "IPMI Management Utilities depend on an IPMI driver, aborting."
        # exit 1
     fi
  fi
fi

if [ ! -d /usr/share/man ]
then
    if [ -d /usr/man ]
    then
	ln -s /usr/man /usr/share/man 
    fi
fi

%post
# after install
pdir=/usr/sbin
tmped=/tmp/edmk.tmp
tmpsel=/tmp/pefcfg.tmp
datadir=/usr/share/ipmiutil
mibdir=/usr/share/snmp/mibs

echo "ipmiutil install started `date`" 
# Assumes that the kernel patches are already in place.

# Make sure that the panic timeout is set to some reasonable value
pnew=5
pfil=/proc/sys/kernel/panic
lilofil=/etc/lilo.conf
pval=`cat $pfil`

if [ ! -f $lilofil ]
then
   # They must have grub instead of lilo, so skip this change. 
   pval="1"
fi
# Always skip this change for ipmiutil
pval="1"

# assume that if $pval is set, that someone has already configured lilo.conf
if [ $pval = "0" ]
then
   # Panic timeout is not set, set timeout to $pnew
   echo "$pnew" >$pfil
   cp -f $lilofil /tmp/lilo.conf-orig
   grep append $lilofil >/dev/null 2>&1
   if [ $? -eq 0 ]
   then
	# modify the append line to add panic timeout
        # can't use $pnew within sed, so set literal 5
        sed -e 's/append=\"/append="panic=5 /' $lilofil >$tmped 
	cp -f $tmped $lilofil
	# use this method in case something else (console) is also in append
   else
	# not found, so add the append line
	cat - <<%%% >$tmped 
1
/timeout
a
append="panic=$pnew"
.
w
q
%%%
	 ed $lilofil <$tmped >/dev/null
   fi
fi

# Check location of snmpd.conf for pefconfig
if [ -f /etc/SuSE-release ]
then
   if [ -f /etc/ucdsnmpd.conf ]
   then
	mkdir -p /etc/snmp
	cd /etc/snmp
	ln -s /etc/ucdsnmpd.conf  snmpd.conf
   fi
   if [ -f /etc/snmpd.conf ]
   then
	mkdir -p /etc/snmp
	cd /etc/snmp
	ln -s /etc/snmpd.conf  snmpd.conf
   fi
fi
if [ -d $mibdir ]
then
   cp -f $datadir/bmclanaol.mib $mibdir/BMCLAN-AOL-MIB.txt
   cp -f $datadir/bmclanpet.mib $mibdir/BMCLAN-PET-MIB.txt
fi

# Use our copy of FreeIPMI library if there isn't one.
if [ ! -f /usr/lib/libfreeipmi.so.0 ]
then
   cd /usr/lib
   ln -s /usr/share/ipmiutil/libfreeipmi.so libfreeipmi.so.0
fi
keydir=/var/lib/freeipmi
keyfil=$keydir/ipckey
if [ ! -f $keyfil ]
then
   mkdir -p $keydir
   touch $keyfil
   chmod 664 $keyfil
fi

# There can be any one of several IPMI driver modules installed.
# Dont need to load a kernel module here. 

# Set up the PEF entry to send the BMC LAN Alert for this event.
$pdir/pefconfig >$tmpsel 2>&1

if [ $? -eq 0 ]
then
  # if pefconfig ran ok (ipmi is supported), then
  # set up a cron job for showsel
  cp /usr/share/ipmiutil/checksel /etc/cron.daily
fi

echo "done `date`" 

%preun
# before uninstall
echo "Uninstalling ipmiutil %Version feature ..."

%postun
# after uninstall
# arg1 = 1 if rpm -U, arg1 = 0 if rpm -e
datadir=/usr/share/ipmiutil
mibdir=/usr/share/snmp/mibs
tmped=/tmp/edmk.tmp

if [ "$1" = "0" ]
then
  # doing rpm -e postun, so clean up anything left over
  rm -rf $datadir  2>/dev/null

  if [ -f /etc/cron.daily/checksel ]
  then
     rm -f /etc/cron.daily/checksel 2>/dev/null
  fi

  if [ -d $mibdir ]
  then
     rm -f $mibdir/BMCLAN-AOL-MIB.txt
     rm -f $mibdir/BMCLAN-PET-MIB.txt
  fi

  ls -l /usr/lib/libfreeipmi.so.0 |grep ipmiutil >/dev/null 2>&1
  if [ $? -eq 0 ]
  then
	# ipmiutil installed the freeipmi, so remove it
	rm -f /usr/lib/libfreeipmi.so.0 
	rm -rf /var/lib/freeipmi
  fi
fi

%changelog
* Wed Feb 03 2005 Andrew Cress <arcress@users.sourceforge.net> 
  changed /usr/man to /usr/share/man,
  fixed %postun to recognize rpm -U via $1 
* Mon Nov 1 2004 Andrew Cress <arcress@users.sourceforge.net> 
  added freeipmi install files and logic
* Tue Aug 23 2004 Andrew Cress <arcress@users.sourceforge.net> 
- added MIB links to /usr/share/snmp/mibs
* Tue Aug 10 2004 Andrew Cress <arcress@users.sourceforge.net> 
- added icmd utility to the rpm
* Thu Aug 05 2004 Andrew Cress <arcress@users.sourceforge.net> 
- added special logic for SuSE snmpd.conf
* Fri Apr 02 2004 Andrew Cress <arcress@users.sourceforge.net> 
- added checksel cron job
* Tue Jan 28 2003 Andrew Cress <arcress@users.sourceforge.net> 
- added sensor & fruconfig for ipmiutil 1.2.8
* Tue Aug  2 2002 Andrew Cress <andrew.r.cress@intel.com> 
- fixed bug 793 (dont need Require:ipmidrvr) for ipmiutil 1.2.2
* Tue Jul  2 2002 Andrew Cress <andrew.r.cress@intel.com> 
- fixed bug 555 in showsel for ipmiutil 1.2.1
* Fri May 10 2002 Andrew Cress <andrew.r.cress@intel.com> 
- fixed bug 504 in pefconfig for ipmiutil 1.1.5
* Thu Apr 11 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated pathnames for ipmiutil 1.1.4, some cleanup
* Mon Mar 18 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.1.3-2, added checking for grub vs. lilo to .spec
* Tue Mar 12 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.1.3, added source rpm, changed license, etc.
* Thu Jan 31 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.1.0-2, changed selpef to pefconfig
* Thu Jan 25 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.1.0, changed to ipmidrvr rather than isc dependency
* Thu Jan 16 2002 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.1.0, added hwreset utility
* Thu Dec 14 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 1.0.0, man page updates
* Thu Nov 19 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 0.9.0, uses new OSS bmc_panic, so don't install module.
* Thu Nov 13 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 0.8.5, add "Requires: isc" (#32), hide selpef output (#38)
* Thu Nov  8 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 0.8.4, eliminate "file exists" messages by fixing removal
* Thu Oct 25 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 0.8.2, run selpef (objdump:applypatch gives bogus warning)
* Thu Oct 25 2001 Andrew Cress <andrew.r.cress@intel.com> 
- updated for 0.8.2, run selpef (objdump:applypatch gives bogus warning)
* Wed Oct 24 2001 Andrew Cress <andrew.r.cress@intel.com> 
- created ipmiutil package 0.8.1 without kbuild
* Tue Oct 23 2001 Andrew Cress <andrew.r.cress@intel.com> 
- created ipmiutil package 0.8.0
